(**********************************************************************
*                                                                     *
*               ꥪ   ᮧ   ᮢ       *
*                                                                     *
**********************************************************************)
{$O-}
                             unit  WinProcs;
                               interface

uses  Objects, Drivers, Views, Menus, TextView, Dialogs, App,
      Figaro, FigaroIO, Commands;
{}
type

  PWinProcess =^TWinProcess;
  TWinProcess =object (TWindow)
      _NamePtr : PString;
      constructor Init  (R:TRect; const Header: string;
                         Process : PProcess);
      destructor  Done  ; virtual;
      procedure   HandleEvent (var E : TEvent); virtual;
  end;

  PProcessTerm = ^TProcessTerm;
  TProcessTerm = object (TProcess)
      procedure Suspend (Time : integer); virtual;
      procedure Resume ; virtual;
      procedure Stop   ; virtual;
  end;

  PWinTerminal =^TWinTerminal;
  TWinTerminal =object (TWinProcess)
      _File : Text;
      _Term : PTerminal;
      constructor Init  (R:TRect; const Header: string;
                         Process : PProcessTerm);
      destructor  Done  ; virtual;
      procedure   Reset;
      procedure   Write   (const s : string);
      procedure   Writeln (const s : string);
  end;

procedure  StartPause;   {  ⠢   DeskTop }

{}
                             implementation
{}
                   {  ⠢   DeskTop }

procedure  StartPause;
var P : PView;
begin
  P:= PView(Get_MyParamPtr);
  repeat   Wait;  until  P^.Owner <> nil;
  Figaro.Pause(CSecond div 2);
end;
{
       ķ
                             TWinProcess                        
       Ľ
}
constructor TWinProcess.Init  (R:TRect; const Header: string;
                               Process : PProcess);
var     res: word;
begin
  if not Assigned(Process) then Fail;
  if Figaro.LowMem or not inherited Init (R, Header, 0)
     then begin
            Dispose(Process, Done);
            Fail;
          end;
  Options:= Options or ofTileable;
  _NamePtr:=NewStr(Process^.GetName);
  Process^._ParamPtr:=@Self;
  res:=Insert_Process (Process);
  if res<> OK then begin
     DisposeStr(_NamePtr);
     Dispose(Process, Done);
     _NamePtr:=nil;
     Fail
  end;
end;

destructor  TWinProcess.Done;
begin
   asm
      PUSHF
      CLI
   end;
   if Assigned(_NamePtr) then begin
      Delete_Process(_NamePtr^);
      DisposeStr (_NamePtr);
      _NamePtr:=nil;
   end;
   inherited Done;
   asm  POPF  end;
end;

procedure   TWinProcess.HandleEvent (var E : TEvent);
begin
  inherited HandleEvent (E);
  if (E.What and evMessage) <> 0 then
     case E.Command of
      cmLock   : Lock_Process    (_NamePtr^);
      cmUnLock : UnLock_Process  (_NamePtr^);
      cmSuspend: Suspend_Process (_NamePtr^, CSecond * 15);
      cmResume : Resume_Process  (_NamePtr^);
      cmStop   : Stop_Process    (_NamePtr^);
      cmRestart: Restart_Process (_NamePtr^);
      cmDelete : Close;
      else  Exit;
     end;
  ClearEvent(E);
end;
{
       ķ
                               TWinTerminal                     
       Ľ
}
constructor TWinTerminal.Init  (R:TRect; const Header: string;
                                Process : PProcessTerm);
var  HSB,VSB  : PScrollBar;
begin
  if not Inherited Init(R,Header,Process) then Fail;
  GetExtent (R);
  R.Grow(-1,-1);
  HSB:=StandardScrollBar(sbHorizontal or sbHandleKeyboard);
  VSB:=StandardScrollBar(sbVertical or sbHandleKeyboard);
  New (_Term,Init(R, HSB, VSB, 512));
  Reset;
  Insert(_Term);
end;

destructor TWinTerminal.Done;
begin
   System.Close(_File);
   inherited Done;
end;

procedure  TWinTerminal.Reset;
begin
  AssignDevice(_File, _Term);
  Rewrite(_File);
end;

procedure  TWinTerminal.Write   (const s : string);
begin
    System.Write (_File, s);
end;

procedure  TWinTerminal.Writeln (const s : string);
begin
  {$I-}
  System.Writeln (_File, s);
  if IOResult <> OK then Reset;
  {$I+}
end;
{
       ķ
                              TProcessTerm                      
       Ľ
}
procedure TProcessTerm.Suspend (Time : integer);
begin
  if Assigned(_ParamPtr) then
     PWinTerminal(_ParamPtr)^.Writeln('Suspended');
  inherited Suspend (Time);
end;

procedure TProcessTerm.Resume ;
begin
  if Assigned(_ParamPtr) then
     PWinTerminal(_ParamPtr)^.Writeln('Resumed');
  inherited Resume;
end;

procedure TProcessTerm.Stop   ;
begin
  if Assigned(_ParamPtr) then
     PWinTerminal(_ParamPtr)^.Writeln('Stopped');
  inherited Stop;
end;

end.
